#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <firestorm.h>
#include <packet.h>
#include <plugin.h>
#include <generator.h>
#include <alert.h>
#include <signature.h>
#include <decode.h>
#include <preproc.h>
#include <netinet/in.h>

PLUGIN_STD_DEFS();
proc_dispatch dispatch;

int http_dprint(struct layer *l, char *buf, int buflen)
{
	struct http_session *h=l->session;
	char *obuf=buf;

	/* 28 = 14+12+2 = extraneous bits of string */
	if ( !h ) return snprintf(buf, buflen, "No session info");
	if ( buflen < 28+h->uri_len+h->method_len )
		return snprintf(buf, buflen, "buffer too small");

	memcpy(buf, "http_method='", 13); buf+=13;
	memcpy(buf, h->method, h->method_len); buf+=h->method_len;
	memcpy(buf, "' http_uri='", 12); buf+=12;
	memcpy(buf, h->uri, h->uri_len); buf+=h->uri_len;
	memcpy(buf, "'", 2);

	return (buf-obuf);
}

int http_serialize(struct packet *p, unsigned int i, char *buf, int len)
{
	struct layer *l=&p->layer[i];
	struct http_session *s=(struct http_session *)l->session;
	struct http_serial  *f=(struct http_serial  *)buf;

	if ( len < sizeof(struct http_serial) )
		return -1;

	f->uri=htonl(s->uri - (char *)l->h.raw);
	f->uri_len=htonl(s->uri_len);
	f->method=htonl(s->method - (char *)l->h.raw);
	f->method_len=htonl(s->method_len);

	return sizeof(*f);
}

/* Remember: this data is un-trusted so validate it! */
void *http_deserialize(struct packet *p, unsigned int i, char *buf, int len)
{
	struct http_session *s;
	struct http_serial *f=(struct http_serial *)buf;
	struct layer *l=&p->layer[i];
	u_int32_t plen;

	if ( len < sizeof(*f) ) return NULL;

	plen=(char *)p->end - (char *)l->h.raw;

	/* validate the input */
	if ( plen < htonl(f->uri)+htonl(f->uri_len) )
		return NULL;
	if ( plen < htonl(f->method)+htonl(f->method_len) )
		return NULL;

	if ( !(s=calloc(1, sizeof(*s))) ) return NULL;

	s->uri=l->h.raw + ntohl(f->uri);
	s->uri_len=ntohl(f->uri_len);
	s->method=l->h.raw + ntohl(f->method);
	s->method_len=ntohl(f->method_len);

	return s;
}

void http_decode(struct packet *p) 
{
	struct pkt_iphdr *iph;
	struct pkt_tcphdr *tcph;
	u_int32_t flags=p->layer[p->llen-1].flags;
	char *ptr=p->layer[p->llen].h.raw;
	char *end;
	size_t rlen;
	size_t plen;
	size_t len;
	int state;
	struct http_session h;

	if ( p->layer[p->llen].h.raw >= p->end )
		goto done;

	h.uri=NULL;
	h.uri_len=0;
	h.method=ptr;
	h.method_len=0;

	/* No special requirements */
	p->llen++;

	/* Only parse HTTP requests requests */
	if ( !(flags & FLAG_TCP_2SVR) ) goto done;

	/* make sure we actually have a payload */
	iph=p->layer[p->llen-3].h.ip;
	tcph=p->layer[p->llen-2].h.tcp;
	rlen=ntohs(iph->tot_len) - ((iph->ihl<<2) + (tcph->doff<<2));
	if ( !rlen ) goto done;

	/* work out min length to scan */
	plen=(char *)p->end-ptr;
	len = ( plen<rlen ) ? plen : rlen;

	/* Really crap way of parsing out URIs, will
	 * fuck up as soon as we see a packet that
	 * doesn't start with an http request */
	for(state=0,end=ptr+len; ptr<end; ptr++) {
		if ( *ptr<32 || *ptr&0x80 ) {
			break;
		}
		if ( state==0 ) {
			if ( *ptr==' ' ) {
				state=1;
				continue;
			}
			h.method_len++;
		}else if ( state==1 ) {
			if ( *ptr!=' ' ) {
				h.uri=ptr;
				h.uri_len=1;
				state=2;
			}
		}else if ( state==2 ) {
			if ( *ptr==' ' ) break;
			h.uri_len++;
		}
	}

	if ( h.uri && h.uri_len && h.method_len ) {
		p->layer[p->llen-1].session=&h;
	}

done:
	dispatch(p);
	return;
}

int _PLUGIN_INIT (struct plugin_in *in, struct plugin_out *out)
{
	static struct proto http_p={
		.name="http",
		.decode=http_decode,
		.print=http_dprint,
		.sflags=PSF_REQUEST,
		.serialize=http_serialize,
		.deserialize=http_deserialize,
		.free=free,
	};
	static struct proto_req http_r[]={
		proto_request("tcp", 80),
		proto_request("tcp", 3128),
		proto_request("tcp", 8080),
		null_request()
	};
	proc_decode_add decode_add;
	int ok=0;
	
	plugin_check(in, out);
	
	PLUGIN_ID("decode.http", "Hyper-text Transfer Protocol");
	PLUGIN_VERSION(0, 1);
	PLUGIN_AUTHOR("Gianni Tedesco", "gianni@scaramanga.co.uk");
	PLUGIN_LICENSE("GPL");
	
	if ( !(decode_add=in->import("decode.add")) ) {
		return PLUGIN_ERR_OBJECT;
	}
	if ( !(dispatch=in->import("dispatch")) ) {
		return PLUGIN_ERR_OBJECT;
	}

	ok+=decode_add(&http_p, http_r);
	
	return (ok) ? PLUGIN_ERR_OK : PLUGIN_ERR_FAIL;
}

int _PLUGIN_UNLOAD (int code) {
	return PLUGIN_ERR_OK;
}
