#ifndef __PREPROC_HEADER_INCLUDED__
#define __PREPROC_HEADER_INCLUDED__

/* We call this with the config args, and the plugin
 * adds a preprocessor function on to the active list */
typedef int (*proc_preproc_init)(char *);

/* A preprocessor */
struct preproc {
	struct preproc		*next;
	const char		*name;
	proc_preproc_init	init;
};

#define PREPROC_OK 0

/* Actually does packet preprocessing */
typedef int (*proc_preprocess)(struct packet *, unsigned int, void *);

/* Frees up the private data */
typedef void (*proc_preproc_free)(void *);

/* An active preprocessor instance */
struct pp_active {
	void *priv;
	proc_preprocess process;
	proc_preproc_free free;
};


#ifndef __PLUGIN__
void preproc_init(void);
void preproc_cleanup(int, void *);
struct preproc *preproc_find(const char *);
int preproc_load(char *, char *);
#endif

#ifndef __PLUGIN__
int preproc_add(const char *, proc_preproc_init);
int preproc_activate(char *, proc_preprocess, proc_preproc_free, void *);
void dispatch(struct packet *);
#else
typedef int (*proc_preproc_add)(const char *, proc_preproc_init);
typedef int (*proc_preproc_activate)(char *, proc_preprocess, proc_preproc_free, void *);
typedef void (*proc_dispatch)(struct packet *);

static inline void preproc_run(struct proto *p, struct packet *pkt)
{
	if ( p->num_active ) {
		int i;
		for(i=0; i<p->num_active; i++) {
			struct pp_active *pp=&p->preproc[i];
			pp->process(pkt, pkt->llen-1, pp->priv);
		}
	}
}

#endif

#endif /* __PREPROC_HEADER_INCLUDED__ */
